@@ -3,7 +3,7 @@ our $AUTHORITY = 'cpan:GENE';
 
 # ABSTRACT: Glorified metronome
 
-our $VERSION = '0.4000';
+our $VERSION = '0.4001';
 
 use Data::Dumper::Compact qw(ddc);
 use List::Util qw(sum0);
@@ -456,7 +456,7 @@ sub add_fill {
         };
     };
     my $fill_patterns = $fill->($self);
-    print 'Fills: ', ddc($fill_patterns) if $self->verbose;
+    print 'Fill: ', ddc($fill_patterns) if $self->verbose;
     my $fill_duration = delete $fill_patterns->{duration} || 8;
     my $fill_length   = length((values %$fill_patterns)[0]);
 
@@ -477,7 +477,7 @@ sub add_fill {
     for my $instrument (keys %patterns) {
         # get a single "flattened" pattern as an arrayref
         my $pattern = [ map { split //, $_ } @{ $patterns{$instrument} } ];
-        # the fresh pattern is upsized with the LCM
+        # the fresh pattern is possibly upsized with the LCM
         $fresh_patterns{$instrument} = @$pattern < $lcm
             ? [ join '', @{ upsize($pattern, $lcm) } ]
             : [ join '', @$pattern ];
@@ -486,26 +486,30 @@ sub add_fill {
 
     my %replacement;
     for my $instrument (keys %$fill_patterns) {
+        # get a single "flattened" pattern as a zero-pre-padded arrayref
         my $pattern = [ split //, sprintf '%0*s', $fill_duration, $fill_patterns->{$instrument} ];
-        my $fresh;
-        if (@$pattern < $lcm) {
-            $fresh = join '', @{ upsize($pattern, $lcm) };
-        }
-        else {
-            $fresh = join '', @$pattern;
-        }
+        # the fresh pattern string is possibly upsized with the LCM
+        my $fresh = @$pattern < $lcm
+            ? join '', @{ upsize($pattern, $lcm) }
+            : join '', @$pattern;
+        # the replacement string is the tail of the fresh pattern string
         $replacement{$instrument} = substr $fresh, -$fill_chop;
     }
     print 'Replacements: ', ddc(\%replacement) if $self->verbose;
 
+    my %replaced;
     for my $instrument (keys %fresh_patterns) {
+        # get the string to replace
         my $string = join '', @{ $fresh_patterns{$instrument} };
+        # replace the tail of the string
         my $pos = length $replacement{$instrument};
         substr $string, -$pos, $pos, $replacement{$instrument};
         print "$instrument: $string\n" if $self->verbose;
+        # prepare the replaced pattern for syncing
+        $replaced{$instrument} = [ $string ];
     }
 
-    $self->sync_patterns(%fresh_patterns);
+    $self->sync_patterns(%replaced);
 }
 
 
@@ -533,16 +537,15 @@ sub write {
     $self->score->write_score( $self->file );
 }
 
+# lifted from https://www.perlmonks.org/?node_id=56906
 sub _gcf {
     my ($x, $y) = @_;
     ($x, $y) = ($y, $x % $y) while $y;
     return $x;
 }
-
 sub _lcm {
     return($_[0] * $_[1] / _gcf($_[0], $_[1]));
 }
-
 sub _multilcm {
     my $x = shift;
     $x = _lcm($x, shift) while @_;
@@ -563,7 +566,7 @@ MIDI::Drummer::Tiny - Glorified metronome
 
 =head1 VERSION
 
-version 0.4000
+version 0.4001
 
 =head1 SYNOPSIS
 
